@@ -37,12 +37,17 @@
                 (let ((drv (package-derivation store package system)))
                   ;; Libc's 'debug' output refers to gcc-cross-boot0, but it's
                   ;; hard to avoid, so we tolerate it.  This should be the
-                  ;; only exception.
+                  ;; only exception.  Likewise, 'bash:include' depends on
+                  ;; bootstrap-binaries via its 'Makefile.inc' (FIXME).
                   (filter-map (match-lambda
                                (("debug" . directory)
                                 (if (string=? "glibc" (package-name package))
                                     #f
                                     directory))
+                               (("include" . directory)
+                                (if (string=? "bash" (package-name package))
+                                    #f
+                                    directory))
                                ((_ . directory) directory))
                               (derivation->output-paths drv)))))
               %final-inputs))
