@@ -1,6 +1,11 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013, 2014 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2013, 2014, 2015 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2013, 2014 Andreas Enge <andreas@enge.fr>
+;;; Copyright © 2015 Taylan Ulrich Bayırlı/Kammer <taylanbayirli@gmail.com>
+;;; Copyright © 2015 Paul van der Walt <paul@denknerd.org>
+;;; Copyright © 2015, 2016 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2016 Alex Kost <alezost@gmail.com>
+;;; Copyright © 2016 Marius Bakke <mbakke@fastmail.com>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -21,10 +26,13 @@
   #:use-module (guix download)
   #:use-module (guix packages)
   #:use-module ((guix licenses) #:select (lgpl2.1+ gpl2 gpl2+ gpl3+))
+  #:use-module (guix build-system cmake)
   #:use-module (guix build-system gnu)
   #:use-module (gnu packages)
   #:use-module (gnu packages acl)
+  #:use-module (gnu packages bison)
   #:use-module (gnu packages compression)
+  #:use-module (gnu packages flex)
   #:use-module (gnu packages gettext)
   #:use-module (gnu packages gtk)
   #:use-module (gnu packages man)
@@ -32,7 +40,7 @@
   #:use-module (gnu packages elf)
   #:use-module (gnu packages pkg-config)
   #:use-module (gnu packages readline)
-  #:use-module (gnu packages which)
+  #:use-module (gnu packages base)
   #:use-module (gnu packages perl)
   #:use-module (gnu packages python)
   #:use-module (gnu packages wget)
@@ -41,14 +49,14 @@
 (define-public libcddb
   (package
     (name "libcddb")
-    (version "1.3.0")
+    (version "1.3.2")
     (source (origin
              (method url-fetch)
-             (uri (string-append "mirror://sourceforge/libcddb/libcddb-"
-                                 version ".tar.bz2"))
+             (uri (string-append "mirror://sourceforge/libcddb/libcddb/" version
+                                 "/libcddb-" version ".tar.bz2"))
              (sha256
               (base32
-               "1y8bfy12dwm41m1jahayn3v47dm34fmz7m9cjxyh7xcw6fp3lzaf"))))
+               "0fr21a7vprdyy1bq6s99m0x420c9jm5fipsd63pqv8qyfkhhxkim"))))
     (build-system gnu-build-system)
     (arguments '(#:tests? #f))      ; tests rely on access to external servers
     (home-page "http://libcddb.sourceforge.net/")
@@ -72,22 +80,22 @@ caching facility provided by the library.")
 (define-public libcdio
   (package
     (name "libcdio")
-    (version "0.93")
+    (version "0.94")
     (source (origin
              (method url-fetch)
              (uri (string-append "mirror://gnu/libcdio/libcdio-"
                                  version ".tar.gz"))
              (sha256
               (base32
-               "0xb9km1750ndr7nglgbv1smv03dy8nkcfd9djbzqn3ldzlicswj9"))))
+               "0nh222bnj0hgdic5nvr8l9j194mh5niqy15rypwrdbk6z01wkqln"))))
     (build-system gnu-build-system)
     (inputs
-       `(("ncurses" ,ncurses)
-         ("libcddb" ,libcddb)))
+     `(("ncurses" ,ncurses)
+       ("libcddb" ,libcddb)))
     (native-inputs
      `(("help2man" ,help2man)
        ("pkg-config" ,pkg-config)))
-    (home-page "http://www.gnu.org/software/libcdio/")
+    (home-page "https://www.gnu.org/software/libcdio/")
     (synopsis "CD Input and Control library")
     (description
      "The GNU Compact Disc Input and Control Library (libcdio) is a library
@@ -99,17 +107,38 @@ libcdio-paranoia, a library providing jitter-free and error-free audio
 extraction from CDs.")
     (license gpl3+)))
 
+(define-public libcdio-paranoia
+  (package
+    (name "libcdio-paranoia")
+    (version "10.2+0.93+1")
+    (source (origin
+             (method url-fetch)
+             (uri (string-append "mirror://gnu/libcdio/libcdio-paranoia-"
+                                 version ".tar.bz2"))
+             (sha256
+              (base32
+               "14x4b4jk5b0zvcalrg02y4jmbkmmlb07qfmk5hph9k18b8frn7gc"))))
+    (build-system gnu-build-system)
+    (native-inputs `(("pkg-config" ,pkg-config)))
+    (propagated-inputs `(("libcdio" ,libcdio)))
+    (home-page "http://www.gnu.org/software/libcdio/")
+    (synopsis "Jitter- and error-tolerant CD audio extraction")
+    (description
+     "libcdio-paranoia is an implementation of CD paranoia libraries based on
+libcdio.")
+    (license gpl3+)))
+
 (define-public xorriso
   (package
     (name "xorriso")
-    (version "1.3.8")
+    (version "1.4.6")
     (source (origin
              (method url-fetch)
              (uri (string-append "mirror://gnu/xorriso/xorriso-"
                                  version ".tar.gz"))
              (sha256
               (base32
-               "0zhhj9lr9z7hnb2alac54mc28w1l0mbanphhpmy3ylsi8rih84lh"))))
+               "112p0ghwzxrcjbsir1n2jxhq103ckrw93wzvd55qqvzfgs674vsj"))))
     (build-system gnu-build-system)
     (inputs
      `(("acl" ,acl)
@@ -117,11 +146,11 @@ extraction from CDs.")
        ("bzip2" ,bzip2)
        ("zlib" ,zlib)
        ("libcdio" ,libcdio)))
-    (home-page "http://www.gnu.org/software/xorriso/")
-    (synopsis "Create, manipulate, burn ISO-9660 filesystems")
+    (home-page "https://www.gnu.org/software/xorriso/")
+    (synopsis "Create, manipulate, burn ISO-9660 file systems")
     (description
      "GNU Xorriso is a tool for copying files to and from ISO 9660 Rock
-Ridge, a.k.a. Compact Disc File System, filesystems and it allows
+Ridge, a.k.a. Compact Disc File System, file systems and it allows
 session-wise manipulation of them.  It features a formatter and burner for
 CD, DVD and BD.  It can operate on existing ISO images or it can create new
 ones.  xorriso can then be used to copy files directly into or out of ISO
@@ -134,35 +163,23 @@ files.")
     (version "10.2")
     (source (origin
              (method url-fetch)
-             (uri (string-append "http://downloads.xiph.org/releases/cdparanoia/cdparanoia-III-"
+             (uri (string-append "http://downloads.xiph.org/releases/"
+                                 "cdparanoia/cdparanoia-III-"
                                  version ".src.tgz"))
              (sha256
               (base32
                "1pv4zrajm46za0f6lv162iqffih57a8ly4pc69f7y0gfyigb8p80"))
-             (patches (list (search-patch "cdparanoia-fpic.patch")))))
+             (patches (search-patches "cdparanoia-fpic.patch"))
+             (modules '((guix build utils)))
+             (snippet
+              ;; Make libraries respect LDFLAGS.
+              '(substitute* '("paranoia/Makefile.in" "interface/Makefile.in")
+                 (("-Wl,-soname") "$(LDFLAGS) -Wl,-soname")))))
     (build-system gnu-build-system)
-    (inputs
-     `(("patchelf" ,patchelf)))
     (arguments
      `(#:tests? #f ; there is no check target
-       #:modules ((guix build gnu-build-system)
-                  (guix build utils)
-                  (guix build rpath)
-                  (srfi srfi-26))
-       #:imported-modules ((guix build gnu-build-system)
-                           (guix build utils)
-                           (guix build rpath))
-       #:phases
-        (alist-cons-after
-         'strip 'add-lib-to-runpath
-         (lambda* (#:key outputs #:allow-other-keys)
-           (let* ((out (assoc-ref outputs "out"))
-                  (lib (string-append out "/lib")))
-             ;; Add LIB to the RUNPATH of all the executables.
-             (with-directory-excursion out
-               (for-each (cut augment-rpath <> lib)
-                         (find-files "bin" ".*")))))
-         %standard-phases)))
+       #:configure-flags ; Add $libdir to the RUNPATH of all the executables.
+       (list (string-append "LDFLAGS=-Wl,-rpath=" %output "/lib"))))
     (home-page "http://www.xiph.org/paranoia/")
     (synopsis "Audio CD reading utility")
     (description "Cdparanoia retrieves audio tracks from CDDA capable CDROM
@@ -177,33 +194,26 @@ reconstruction capability.")
 (define-public dvdisaster
   (package
     (name "dvdisaster")
-    (version "0.72.6")
+    (version "0.79.5")
     (source (origin
              (method url-fetch)
              (uri (string-append "http://dvdisaster.net/downloads/dvdisaster-"
                                  version ".tar.bz2"))
              (sha256
               (base32
-               "0sqrprc5rh3shnfli25m2wy0i5f83db54iv04s5s7bxf77m7sy79"))))
+               "0f8gjnia2fxcbmhl8b3qkr5b7idl8m855dw7xw2fnmbqwvcm6k4w"))))
     (build-system gnu-build-system)
     (inputs
      `(("gtk+" ,gtk+-2)))
     (native-inputs
-     `(("gettext" ,gnu-gettext)
+     `(("gettext" ,gettext-minimal)
        ("pkg-config" ,pkg-config)
        ("which" ,which)))
     (arguments
      `(;; Parallel builds appear to be unsafe, see
        ;; <http://hydra.gnu.org/build/49331/nixlog/1/raw>.
        #:parallel-build? #f
-       #:tests? #f ; no check target
-       #:phases
-         (alist-cons-before
-          'patch-source-shebangs 'sanitise
-          (lambda _
-            ;; delete dangling symlink
-            (delete-file ".#GNUmakefile"))
-          %standard-phases)))
+       #:tests? #f)) ; no check target
     (home-page "http://dvdisaster.net/en/index.html")
     (synopsis "Error correcting codes for optical media images")
     (description "Optical media (CD,DVD,BD) keep their data only for a
@@ -222,6 +232,30 @@ depend on the file system of the medium.  The maximum error correction
 capacity is user-selectable.")
     (license gpl2+)))
 
+(define-public libcue
+  (package
+    (name "libcue")
+    (version "2.1.0")
+    (source (origin
+             (method url-fetch)
+             (uri (string-append
+                   "https://github.com/lipnitsk/libcue/archive/v"
+                   version ".tar.gz"))
+             (file-name (string-append name "-" version ".tar.gz"))
+             (sha256
+              (base32
+               "1fradl3dx0pyy9rn1a0gak9gzgg40wax61f2s00zks7rwl0xv398"))))
+    (build-system cmake-build-system)
+    (native-inputs
+     `(("bison" ,bison)
+       ("flex" ,flex)))
+    (home-page "https://github.com/lipnitsk/libcue")
+    (synopsis "C library to parse cue sheets")
+    (description "Libcue is a C library to parse so-called @dfn{cue sheets}
+which contain meta-data for CD/DVD tracks.  It provides an API to manipulate
+the data.")
+    (license gpl2+)))
+
 (define-public cd-discid
   (package
     (name "cd-discid")
@@ -255,15 +289,15 @@ from an audio CD.")
 (define-public abcde
   (package
     (name "abcde")
-    (version "2.6")
-    (home-page "http://abcde.einval.com/")
+    (version "2.7.2")
+    (home-page "https://abcde.einval.com/")
     (source (origin
               (method url-fetch)
               (uri (string-append home-page "/download/abcde-"
                                   version ".tar.gz"))
               (sha256
                (base32
-                "0y2cg233n2hixs0ji76dggpzgf52v4c4mnpwiai889ql2piafgk8"))
+                "1pakpi41k8yd780mfp0snhia6mmwjwxk9lcrq6gynimch8b8hfda"))
               (modules '((guix build utils)))
               (snippet
                '(substitute* "Makefile"
@@ -273,38 +307,42 @@ from an audio CD.")
                    (string-append "etcdir = $(prefix)/etc\n"))))))
     (build-system gnu-build-system)
     (arguments
-     '(#:phases (alist-replace
-                 'configure
-                 (lambda* (#:key outputs inputs #:allow-other-keys)
-                   (substitute* "Makefile"
-                     (("^prefix = .*$")
-                      (string-append "prefix = "
-                                     (assoc-ref outputs "out")
-                                     "\n"))))
-                 (alist-cons-after
-                  'install 'wrap
-                  (lambda* (#:key inputs outputs #:allow-other-keys)
-                    (let ((wget   (assoc-ref inputs "wget"))
-                          (vorbis (assoc-ref inputs "vorbis-tools"))
-                          (parano (assoc-ref inputs "cdparanoia"))
-                          (which  (assoc-ref inputs "which"))
-                          (discid (assoc-ref inputs "cd-discid"))
-                          (out    (assoc-ref outputs "out")))
-                      (define (wrap file)
-                        (wrap-program file
-                                      `("PATH" ":" prefix
-                                        (,(string-append out "/bin:"
-                                                         wget "/bin:"
-                                                         which "/bin:"
-                                                         vorbis "/bin:"
-                                                         discid "/bin:"
-                                                         parano "/bin")))))
+     '(#:phases
+       (modify-phases %standard-phases
+         (replace 'configure
+           (lambda* (#:key outputs inputs #:allow-other-keys)
+             (substitute* "Makefile"
+               (("^prefix = .*$")
+                (string-append "prefix = "
+                               (assoc-ref outputs "out")
+                               "\n"))
+               (("^sysconfdir = .*$")
+                (string-append "sysconfdir = "
+                               (assoc-ref outputs "out")
+                               "/etc/\n")))
+             #t))
+         (add-after 'install 'wrap
+           (lambda* (#:key inputs outputs #:allow-other-keys)
+             (let ((wget   (assoc-ref inputs "wget"))
+                   (vorbis (assoc-ref inputs "vorbis-tools"))
+                   (parano (assoc-ref inputs "cdparanoia"))
+                   (which  (assoc-ref inputs "which"))
+                   (discid (assoc-ref inputs "cd-discid"))
+                   (out    (assoc-ref outputs "out")))
+               (define (wrap file)
+                 (wrap-program file
+                               `("PATH" ":" prefix
+                                 (,(string-append out "/bin:"
+                                                  wget "/bin:"
+                                                  which "/bin:"
+                                                  vorbis "/bin:"
+                                                  discid "/bin:"
+                                                  parano "/bin")))))
 
-                      (for-each wrap
-                                (find-files (string-append out "/bin")
-                                            ".*"))))
-                  %standard-phases))
-       #:tests? #f))
+               (for-each wrap
+                         (find-files (string-append out "/bin")
+                                     ".*"))))))
+       #:tests? #f)) ; no test target
 
     (inputs `(("wget" ,wget)
               ("which" ,which)
@@ -322,3 +360,38 @@ from an audio CD.")
 that grabs tracks off a CD, encodes them to Ogg/Vorbis, MP3, FLAC, Ogg/Speex
 and/or MPP/MP+ (Musepack) format, and tags them, all in one go.")
     (license gpl2+)))
+
+(define-public geteltorito
+  (package
+    (name "geteltorito")
+    (version "0.6")
+    (home-page
+     "https://userpages.uni-koblenz.de/~krienke/ftp/noarch/geteltorito/")
+    (source (origin
+              (method url-fetch)
+              (uri (string-append home-page name "-" version ".tar.gz"))
+              (sha256
+               (base32
+                "1gkbm9ahj2mgqrkrfpibzclsriqgsbsvjh19fr815vpd9f6snkxv"))))
+    (build-system gnu-build-system)
+    (arguments
+     `(#:tests? #f ; No tests.
+       #:phases
+       (modify-phases %standard-phases
+         (delete 'configure)
+         (delete 'build)
+         (replace 'install
+           (lambda* (#:key outputs #:allow-other-keys)
+             (let ((out (assoc-ref outputs "out")))
+               (install-file "geteltorito"
+                             (string-append out "/bin"))))))))
+    (inputs `(("perl" ,perl)))
+    (synopsis "Extract the boot image from a CD-ROM")
+    (description
+     "@command{geteltorito} can extract the initial/default boot
+image from CDs (and ISOs) that follow the El Torito specification
+for bootable CD-ROMs.
+
+Image data is written to standard output by default and all other
+information is written to standard error.")
+    (license gpl2+)))
