@@ -1,6 +1,7 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2012 Nikita Karetnikov <nikita@karetnikov.org>
-;;; Copyright © 2014 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2014, 2015 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2016 Efraim Flashner <efraim@flashner.co.il>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -19,11 +20,12 @@
 
 (define-module (gnu packages wget)
   #:use-module (guix licenses)
-  #:use-module (gnu packages gnutls)
   #:use-module (gnu packages libidn)
   #:use-module (gnu packages python)
   #:use-module (gnu packages perl)
   #:use-module (gnu packages web)
+  #:use-module (gnu packages pkg-config)
+  #:use-module (gnu packages tls)
   #:use-module (guix packages)
   #:use-module (guix download)
   #:use-module (guix build-system gnu))
@@ -31,7 +33,7 @@
 (define-public wget
   (package
     (name "wget")
-    (version "1.16.1")
+    (version "1.18")
     (source
      (origin
       (method url-fetch)
@@ -39,14 +41,15 @@
                           version ".tar.xz"))
       (sha256
        (base32
-        "0csdw41hixa4kd0m19r7p41sip1hlnkp5y62bdzq9zhmxq3wg5ib"))))
+        "1hcwx8ww3sxzdskkx3l7q70a7wd6569yrnjkw9pw013cf9smpddm"))))
     (build-system gnu-build-system)
     (inputs
      `(("gnutls" ,gnutls)
        ("libidn" ,libidn)
        ("libpsl" ,libpsl)))
     (native-inputs
-     `(("perl" ,perl)
+     `(("pkg-config" ,pkg-config)
+       ("perl" ,perl)
        ("python" ,python)               ;for testenv suite
        ("perl-http-daemon" ,perl-http-daemon)
        ("perl-io-socket-ssl" ,perl-io-socket-ssl)))
@@ -54,7 +57,40 @@
     (synopsis "Non-interactive command-line utility for downloading files")
     (description
      "GNU Wget is a non-interactive tool for fetching files using the HTTP,
-HTTPS and FTP protocols.  It can resume interrupted downloads, use filename
+HTTPS and FTP protocols.  It can resume interrupted downloads, use file name
 wild cards, supports proxies and cookies, and it can convert absolute links
 in downloaded documents to relative links.")
     (license gpl3+))) ; some files are under GPLv2+
+
+(define-public wgetpaste
+  (package
+    (name "wgetpaste")
+    (version "2.28")
+    (source
+      (origin
+        (method url-fetch)
+        (uri (string-append "http://wgetpaste.zlin.dk/wgetpaste-"
+                            version ".tar.bz2"))
+        (sha256
+         (base32
+          "1hh9svyypqcvdg5mjxyyfzpdzhylhf7s7xq5dzglnm4injx3i3ak"))))
+    (build-system gnu-build-system)
+    (arguments
+     '(#:phases
+       (modify-phases %standard-phases
+         (delete 'configure)
+         (delete 'build)
+         (replace 'install
+           (lambda* (#:key outputs #:allow-other-keys)
+             (let* ((out (assoc-ref outputs "out"))
+                    (bin (string-append out "/bin"))
+                    (zsh (string-append out "/share/zsh/site-functions")))
+               (install-file "wgetpaste" bin)
+               (install-file "_wgetpaste" zsh)))))
+       #:tests? #f)) ; no test target
+    (home-page "http://wgetpaste.zlin.dk/")
+    (synopsis "Script that automates pasting to a number of pastebin services")
+    (description
+     "@code{wgetpaste} is an extremely simple command-line interface to various
+online pastebin services.")
+    (license public-domain)))
