@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2013, 2016 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2016 Efraim Flashner <efraim@flashner.co.il>
 ;;;
 ;;; This file is part of GNU Guix.
@@ -23,6 +23,7 @@
   #:use-module (guix licenses)
   #:use-module (guix download)
   #:use-module (gnu packages)
+  #:use-module (gnu packages gcc)
   #:use-module (gnu packages tcl))
 
 (define-public wordnet
@@ -51,7 +52,7 @@
 
                                ;; Provide the `result' field in `Tcl_Interp'.
                                ;; See <https://bugs.gentoo.org/show_bug.cgi?id=452034>.
-                               "CFLAGS=-DUSE_INTERP_RESULT")
+                               "CFLAGS=-DUSE_INTERP_RESULT -O2")
        #:phases
        (modify-phases %standard-phases
          (add-after 'install 'post-install
@@ -78,12 +79,18 @@
                #t))))))
     (outputs '("out"
                "tk"))                             ; for the Tcl/Tk GUI
+
+    ;; Build with a patched GCC to work around <http://bugs.gnu.org/24703>.
+    ;; (Specifically the 'DEFAULTPATH' string literal is what we want to
+    ;; prevent from being chunked so that grafting can "see" it and patch it.)
+    (native-inputs `(("gcc@6" ,gcc-6)))
+
     (inputs `(("tk" ,tk)
               ("tcl" ,tcl)))
     (home-page "http://wordnet.princeton.edu/")
     (synopsis "Lexical database for the English language")
     (description
-     "WordNet® is a large lexical database of English.  Nouns, verbs,
+     "WordNet is a large lexical database of English.  Nouns, verbs,
 adjectives and adverbs are grouped into sets of cognitive synonyms (synsets),
 each expressing a distinct concept.  Synsets are interlinked by means of
 conceptual-semantic and lexical relations.  The resulting network of
