@@ -61,51 +61,62 @@
 (test-assert "cpan->guix-package"
   ;; Replace network resources with sample data.
   (mock ((guix build download) url-fetch
-         (lambda* (url file-name #:key (mirrors '()))
+         (lambda* (url file-name
+                       #:key
+                       (mirrors '()) verify-certificate?)
            (with-output-to-file file-name
              (lambda ()
                (display
                 (match url
-                  ("http://api.metacpan.org/release/Foo-Bar"
-                   test-json)
-                  ("http://api.metacpan.org/module/Test::Script"
-                   "{ \"distribution\" : \"Test-Script\" }")
                   ("http://example.com/Foo-Bar-0.1.tar.gz"
                    test-source)
                   (_ (error "Unexpected URL: " url))))))))
-    (match (cpan->guix-package "Foo::Bar")
-      (('package
-         ('name "perl-foo-bar")
-         ('version "0.1")
-         ('source ('origin
-                    ('method 'url-fetch)
-                    ('uri ('string-append "http://example.com/Foo-Bar-"
-                                          'version ".tar.gz"))
-                    ('sha256
-                     ('base32
-                      (? string? hash)))))
-         ('build-system 'perl-build-system)
-         ('inputs
-          ('quasiquote
-           (("perl-test-script" ('unquote 'perl-test-script)))))
-         ('home-page "http://search.cpan.org/dist/Foo-Bar")
-         ('synopsis "Fizzle Fuzz")
-         ('description 'fill-in-yourself!)
-         ('license (package-license perl)))
-       (string=? (bytevector->nix-base32-string
-                  (call-with-input-string test-source port-sha256))
-                 hash))
-      (x
-       (pk 'fail x #f)))))
+        (mock ((guix http-client) http-fetch
+               (lambda (url)
+                 (match url
+                   ("https://api.metacpan.org/release/Foo-Bar"
+                    (values (open-input-string test-json)
+                            (string-length test-json)))
+                   ("https://api.metacpan.org/module/Test::Script?fields=distribution"
+                    (let ((result "{ \"distribution\" : \"Test-Script\" }"))
+                      (values (open-input-string result)
+                              (string-length result))))
+                   (_ (error "Unexpected URL: " url)))))
+              (match (cpan->guix-package "Foo::Bar")
+                (('package
+                   ('name "perl-foo-bar")
+                   ('version "0.1")
+                   ('source ('origin
+                              ('method 'url-fetch)
+                              ('uri ('string-append "http://example.com/Foo-Bar-"
+                                                    'version ".tar.gz"))
+                              ('sha256
+                               ('base32
+                                (? string? hash)))))
+                   ('build-system 'perl-build-system)
+                   ('inputs
+                    ('quasiquote
+                     (("perl-test-script" ('unquote 'perl-test-script)))))
+                   ('home-page "http://search.cpan.org/dist/Foo-Bar")
+                   ('synopsis "Fizzle Fuzz")
+                   ('description 'fill-in-yourself!)
+                   ('license (package-license perl)))
+                 (string=? (bytevector->nix-base32-string
+                            (call-with-input-string test-source port-sha256))
+                           hash))
+                (x
+                 (pk 'fail x #f))))))
 
 (test-equal "source-url-http"
-  ((@@ (guix import cpan) fix-source-url)
-   "http://cpan.metacpan.org/authors/id/T/TE/TEST/Foo-Bar-0.1.tar.gz")
+  ((@@ (guix import cpan) cpan-source-url)
+   `(("download_url" .
+      "http://cpan.metacpan.org/authors/id/T/TE/TEST/Foo-Bar-0.1.tar.gz")))
   "mirror://cpan/authors/id/T/TE/TEST/Foo-Bar-0.1.tar.gz")
 
 (test-equal "source-url-https"
-  ((@@ (guix import cpan) fix-source-url)
-   "https://cpan.metacpan.org/authors/id/T/TE/TEST/Foo-Bar-0.1.tar.gz")
+  ((@@ (guix import cpan) cpan-source-url)
+   `(("download_url" .
+      "https://cpan.metacpan.org/authors/id/T/TE/TEST/Foo-Bar-0.1.tar.gz")))
   "mirror://cpan/authors/id/T/TE/TEST/Foo-Bar-0.1.tar.gz")
 
 (test-end "cpan")
