@@ -1,5 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2014 David Thompson <davet@gnu.org>
+;;; Copyright © 2016 Ricardo Wurmus <rekado@elephly.net>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -91,51 +92,55 @@ baz > 13.37")
 
 (test-assert "pypi->guix-package"
   ;; Replace network resources with sample data.
-  (mock ((guix import utils) url-fetch
-         (lambda (url file-name)
-           (match url
-             ("https://pypi.python.org/pypi/foo/json"
-              (with-output-to-file file-name
-                (lambda ()
-                  (display test-json))))
-             ("https://example.com/foo-1.0.0.tar.gz"
-               (begin
-                 (mkdir "foo-1.0.0")
-                 (with-output-to-file "foo-1.0.0/requirements.txt"
-                   (lambda ()
-                     (display test-requirements)))
-                 (system* "tar" "czvf" file-name "foo-1.0.0/")
-                 (delete-file-recursively "foo-1.0.0")
-                 (set! test-source-hash
-                       (call-with-input-file file-name port-sha256))))
-             ("https://example.com/foo-1.0.0-py2.py3-none-any.whl" #f)
-             (_ (error "Unexpected URL: " url)))))
-    (match (pypi->guix-package "foo")
-      (('package
-         ('name "python-foo")
-         ('version "1.0.0")
-         ('source ('origin
-                    ('method 'url-fetch)
-                    ('uri (string-append "https://example.com/foo-"
-                                         version ".tar.gz"))
-                    ('sha256
-                     ('base32
-                      (? string? hash)))))
-         ('build-system 'python-build-system)
-         ('inputs
-          ('quasiquote
-           (("python-bar" ('unquote 'python-bar))
-            ("python-baz" ('unquote 'python-baz))
-            ("python-setuptools" ('unquote 'python-setuptools)))))
-         ('home-page "http://example.com")
-         ('synopsis "summary")
-         ('description "summary")
-         ('license 'lgpl2.0))
-       (string=? (bytevector->nix-base32-string
-                  test-source-hash)
-                 hash))
-      (x
-       (pk 'fail x #f)))))
+    (mock ((guix import utils) url-fetch
+           (lambda (url file-name)
+             (match url
+               ("https://example.com/foo-1.0.0.tar.gz"
+                (begin
+                  (mkdir "foo-1.0.0")
+                  (with-output-to-file "foo-1.0.0/requirements.txt"
+                    (lambda ()
+                      (display test-requirements)))
+                  (system* "tar" "czvf" file-name "foo-1.0.0/")
+                  (delete-file-recursively "foo-1.0.0")
+                  (set! test-source-hash
+                    (call-with-input-file file-name port-sha256))))
+               ("https://example.com/foo-1.0.0-py2.py3-none-any.whl" #f)
+               (_ (error "Unexpected URL: " url)))))
+          (mock ((guix http-client) http-fetch
+                 (lambda (url)
+                   (match url
+                     ("https://pypi.python.org/pypi/foo/json"
+                      (values (open-input-string test-json)
+                              (string-length test-json)))
+                     ("https://example.com/foo-1.0.0-py2.py3-none-any.whl" #f)
+                     (_ (error "Unexpected URL: " url)))))
+                (match (pypi->guix-package "foo")
+                  (('package
+                     ('name "python-foo")
+                     ('version "1.0.0")
+                     ('source ('origin
+                                ('method 'url-fetch)
+                                ('uri (string-append "https://example.com/foo-"
+                                                     version ".tar.gz"))
+                                ('sha256
+                                 ('base32
+                                  (? string? hash)))))
+                     ('build-system 'python-build-system)
+                     ('propagated-inputs
+                      ('quasiquote
+                       (("python-bar" ('unquote 'python-bar))
+                        ("python-baz" ('unquote 'python-baz))
+                        ("python-setuptools" ('unquote 'python-setuptools)))))
+                     ('home-page "http://example.com")
+                     ('synopsis "summary")
+                     ('description "summary")
+                     ('license 'license:lgpl2.0))
+                   (string=? (bytevector->nix-base32-string
+                              test-source-hash)
+                             hash))
+                  (x
+                   (pk 'fail x #f))))))
 
 (test-skip (if (which "zip") 0 1))
 (test-assert "pypi->guix-package, wheels"
@@ -143,10 +148,6 @@ baz > 13.37")
   (mock ((guix import utils) url-fetch
          (lambda (url file-name)
            (match url
-             ("https://pypi.python.org/pypi/foo/json"
-              (with-output-to-file file-name
-                (lambda ()
-                  (display test-json))))
              ("https://example.com/foo-1.0.0.tar.gz"
                (begin
                  (mkdir "foo-1.0.0")
@@ -170,31 +171,39 @@ baz > 13.37")
                    (rename-file zip-file file-name))
                  (delete-file-recursively "foo-1.0.0.dist-info")))
              (_ (error "Unexpected URL: " url)))))
-    (match (pypi->guix-package "foo")
-      (('package
-         ('name "python-foo")
-         ('version "1.0.0")
-         ('source ('origin
-                    ('method 'url-fetch)
-                    ('uri (string-append "https://example.com/foo-"
-                                         version ".tar.gz"))
-                    ('sha256
-                     ('base32
-                      (? string? hash)))))
-         ('build-system 'python-build-system)
-         ('inputs
-          ('quasiquote
-           (("python-bar" ('unquote 'python-bar))
-            ("python-baz" ('unquote 'python-baz))
-            ("python-setuptools" ('unquote 'python-setuptools)))))
-         ('home-page "http://example.com")
-         ('synopsis "summary")
-         ('description "summary")
-         ('license 'lgpl2.0))
-       (string=? (bytevector->nix-base32-string
-                  test-source-hash)
-                 hash))
-      (x
-       (pk 'fail x #f)))))
+        (mock ((guix http-client) http-fetch
+               (lambda (url)
+                 (match url
+                   ("https://pypi.python.org/pypi/foo/json"
+                    (values (open-input-string test-json)
+                            (string-length test-json)))
+                   ("https://example.com/foo-1.0.0-py2.py3-none-any.whl" #f)
+                   (_ (error "Unexpected URL: " url)))))
+              (match (pypi->guix-package "foo")
+                (('package
+                   ('name "python-foo")
+                   ('version "1.0.0")
+                   ('source ('origin
+                              ('method 'url-fetch)
+                              ('uri (string-append "https://example.com/foo-"
+                                                   version ".tar.gz"))
+                              ('sha256
+                               ('base32
+                                (? string? hash)))))
+                   ('build-system 'python-build-system)
+                   ('propagated-inputs
+                    ('quasiquote
+                     (("python-bar" ('unquote 'python-bar))
+                      ("python-baz" ('unquote 'python-baz))
+                      ("python-setuptools" ('unquote 'python-setuptools)))))
+                   ('home-page "http://example.com")
+                   ('synopsis "summary")
+                   ('description "summary")
+                   ('license 'license:lgpl2.0))
+                 (string=? (bytevector->nix-base32-string
+                            test-source-hash)
+                           hash))
+                (x
+                 (pk 'fail x #f))))))
 
 (test-end "pypi")
